###########################################################
# Check snapshot profiles of distributed generation (DG)  #
# AMPL coding by                                          #
# Fu Lin                                                  #
# Argonne National Laboratory, 2014                       #
###########################################################

### Check load profiles ###
param CheckEps = 1.e-7;

check {h in 1..24, j in TechsSOFC, k in OnOffCol[j]}: OnOffProf[h,j,k] * (1-OnOffProf[h,j,k]) = 0;
					# ... check if the on-off profiles are binary

check {h in 1..24, j in TechsSOFC, k in OnOffCol[j], l in GenCol[j,k]}: 
	MinPowerRate[j]*OnOffProf[h,j,k] - CheckEps <= GenProf[h,j,k,l] <= MaxPowerRate[j]*OnOffProf[h,j,k] + CheckEps;
					# ... check if the generation profile is within min/max bounds

check {h in 1..24, k in PurCol}: PurProf[h,k] >= 0;
					# ... check if the purchased profiles are non-negative

check {h in 1..24, k in BattCol}: BattProf[h,k] >= 0;
					# ... check if the battery storage profiles are non-negative

check {h in 1..24, k in BattIOCol: BattIOProf[h,k] < 0}: -BattIOProf[h,k] <= BattProf[h,k] + CheckEps;
					# ... check if the output battery profile is upper bounded by the storage profile

					# ... BattIOProf is Input - Output. If it is negative, it means battery is discharging.

check {h in 1..24, k in HeatStorCol}: HeatStorProf[h,k] >= 0;
					# ... check if the heat storage profiles are non-negative

check {h in 1..24, k in HeatStorOutCol}: 0 <= HeatStorOutProf[h,k] <= HeatStorProf[h,k] + CheckEps;
					# ... check if the heat output for storage is non-negative and upper bounded by storage

check {h in 1..24, k in HeatBoilOutCol}: HeatBoilOutProf[h,k] >= 0;
					# ... check if the heat output from boiler is non-negative
