####################################################
# Choose profiles using sampling heuristics        #
# AMPL coding by 				   #
# Fu Lin                                           #
# Argonne National Laboratory, 2014                #
####################################################

# Load the non-redundant on/off profiles into profile pools
#commands LoadProf.ampl;

#option NumDays 364;		# ... the number of days for profiles
#option SamplingRate 14;    	# ... take one sample over SamplingRate samples

# apply selection criteria to generate working profiles

# choose the most frequently used columns that are not all zero
param FreqNum := 3;
set FreqCol{TechsSOFC} default {};
param MaxCard{TechsSOFC};
set MostFreqCol{TechsSOFC} ordered default {};

# choose two production profiles from the pools
param NormOneGenProf{j in TechsSOFC, k in OnOffColPool[j], l in GenColPool[j,k]} := sum{h in 1..24} GenProfPool[h,j,k,l];
set NormOneMaxGenCol{j in TechsSOFC, k in OnOffColPool[j]} ordered 
	:= {l in GenColPool[j,k]: NormOneGenProf[j,k,l] = max{r in GenColPool[j,k]} NormOneGenProf[j,k,r]}; 

set NormOneMinGenCol{j in TechsSOFC, k in OnOffColPool[j]} ordered 
	:= {l in GenColPool[j,k]: NormOneGenProf[j,k,l] = min{r in GenColPool[j,k]} NormOneGenProf[j,k,r]}; 

# create profiles pools for other variables
set BattColPool; 		let BattColPool        := {1..num($NumDays)};
set BattIOColPool; 		let BattIOColPool      := {1..num($NumDays)};
set HeatStorColPool; 		let HeatStorColPool    := {1..num($NumDays)};
set HeatStorOutColPool; 	let HeatStorOutColPool := {1..num($NumDays)};
set PurColPool; 		let PurColPool         := {1..num($NumDays)};
set HeatBoilOutColPool; 	let HeatBoilOutColPool := {1..num($NumDays)};

param PurProfPool{1..24, PurColPool};
param BattProfPool{1..24, BattColPool};
param BattIOProfPool{1..24, BattColPool};
param HeatStorProfPool{1..24, HeatStorColPool};
param HeatStorOutProfPool{1..24, HeatStorColPool};
param HeatBoilOutProfPool{1..24, HeatBoilOutColPool};

# choose profiles out of all possible profiles in the pools
param NormOnePurProf{k in PurColPool} 		  	:= sum{h in 1..24} PurProfPool[h,k];
param NormOneBattProf{k in BattColPool} 		:= sum{h in 1..24} BattProfPool[h,k];
param NormOneHeatStorProf{k in HeatStorColPool} 	:= sum{h in 1..24} HeatStorProfPool[h,k];
param NormOneHeatBoilOutProf{k in HeatBoilOutColPool} 	:= sum{h in 1..24} HeatBoilOutProfPool[h,k];

set NormOneMaxPurCol 	  ordered := {k in PurColPool: NormOnePurProf[k] = max{l in PurColPool} NormOnePurProf[l]}; 
set NormOneMaxBattCol 	  ordered := {k in BattColPool: NormOneBattProf[k] = max{l in BattColPool} NormOneBattProf[l]};
set NormOneMaxHeatStorCol  ordered := {k in HeatStorColPool: NormOneHeatStorProf[k] = 
					max{l in HeatStorColPool} NormOneHeatStorProf[l]};
set NormOneMaxHeatBoilOutCol ordered := {k in HeatBoilOutColPool: NormOneHeatBoilOutProf[k] = max{l in HeatBoilOutColPool} 
					NormOneHeatBoilOutProf[l]};

set NormOneMinPurCol 	  ordered := {k in PurColPool: NormOnePurProf[k] = min{l in PurColPool} NormOnePurProf[l]}; 
set NormOneMinBattCol 	  ordered := {k in BattColPool: NormOneBattProf[k] = min{l in BattColPool} NormOneBattProf[l]};
set NormOneMinHeatStorCol ordered := {k in HeatStorColPool: NormOneHeatStorProf[k] = 
					min{l in HeatStorColPool} NormOneHeatStorProf[l]};
set NormOneMinHeatBoilOutCol ordered := {k in HeatBoilOutColPool: NormOneHeatBoilOutProf[k] = min{l in HeatBoilOutColPool} 
					NormOneHeatBoilOutProf[l]};

	read {d in PurColPool,         h in 1..24} PurProfPool[h,d]         
			< ("snapshots/PowerUtility/"    &   $Building & $NumDays & $MovHor & "day.sns");  
	read {d in HeatBoilOutColPool, h in 1..24} HeatBoilOutProfPool[h,d] 
			< ("snapshots/HeatOut/Boiler"   &   $Building & $NumDays & $MovHor & "day.sns");  
	read {d in BattColPool,        h in 1..24} BattProfPool[h,d]        
			< ("snapshots/PowerStorage/"    &   $Building & $NumDays & $MovHor & "day.sns");  
	read {d in BattIOColPool,      h in 1..24} BattIOProfPool[h,d]      
			< ("snapshots/BattInputOutput/" &   $Building & $NumDays & $MovHor & "day.sns");  
	read {d in HeatStorColPool,    h in 1..24} HeatStorProfPool[h,d]    
			< ("snapshots/HeatStorage/"     &   $Building & $NumDays & $MovHor & "day.sns");  
	read {d in HeatStorOutColPool, h in 1..24} HeatStorOutProfPool[h,d] 
			< ("snapshots/HeatOut/Storage"  &   $Building & $NumDays & $MovHor & "day.sns");

	# choose the most frequently used columns that are not all zero
	for {j in TechsSOFC}{
	    # if the number of OnOffCol is greater than the chosen number of frequent columns
            # then choose the most frequent columns
  	    # else choose all columns in the pool
	    if card(OnOffColPool[j]) >= FreqNum  
	    then{
	        for {r in 1..FreqNum}{
		    # find the on/off column in the pool that has the most generation profiles
		    let MaxCard[j] := max{k in OnOffColPool[j] diff FreqCol[j]: 
					sum{h in 1..24} OnOffProfPool[h,j,k] > 0} 
					card(GenColPool[j,k]);
		    #let MaxCard[j] := max{k in OnOffColPool[j] diff FreqCol[j]} card(GenColPool[j,k]);
		    let MostFreqCol[j] := {k in OnOffColPool[j]: card(GenColPool[j,k]) = MaxCard[j]};	
		    let FreqCol[j] := FreqCol[j] union {first(MostFreqCol[j])};
	        }
		let OnOffCol[j] := FreqCol[j];
	    }    
	    else
		let OnOffCol[j] := OnOffColPool[j];
	}
	display OnOffCol;

	# choose two production profiles 
	let {j in TechsSOFC, k in OnOffCol[j]} GenCol[j,k] := {first(NormOneMaxGenCol[j,k])} union {first(NormOneMinGenCol[j,k])};

	let BattCol 	   := {first(NormOneMaxBattCol)} union {first(NormOneMinBattCol)};
	let BattIOCol 	   := BattCol;
	let HeatStorCol    := {first(NormOneMaxHeatStorCol)} union {first(NormOneMinHeatStorCol)};
	let HeatStorOutCol := HeatStorCol;

	let PurCol         := {1 .. num($NumDays) by num($SamplingRate)};
	let HeatBoilOutCol := {1 .. num($NumDays) by num($SamplingRate)};

############################  Load profiles ##################################

# load on/off profiles
let {h in 1..24, j in TechsSOFC, k in OnOffCol[j]} OnOffProf[h,j,k] := OnOffProfPool[h,j,k];

# load generation profiles
let {h in 1..24, j in TechsSOFC, k in OnOffCol[j], l in GenCol[j,k]} GenProf[h,j,k,l] := GenProfPool[h,j,k,l];

# load other profiles
let {h in 1..24, k in PurCol} 		PurProf[h,k] 	     := PurProfPool[h,k];
let {h in 1..24, k in HeatBoilOutCol} 	HeatBoilOutProf[h,k] := HeatBoilOutProfPool[h,k];
let {h in 1..24, k in BattCol} 		BattProf[h,k] 	     := BattProfPool[h,k];
let {h in 1..24, k in BattIOCol} 	BattIOProf[h,k]      := BattIOProfPool[h,k];
let {h in 1..24, k in HeatStorCol} 	HeatStorProf[h,k]    := HeatStorProfPool[h,k];
let {h in 1..24, k in HeatStorOutCol} 	HeatStorOutProf[h,k] := HeatStorOutProfPool[h,k];

##############################################################################


###########################  Clean up pools ##################################

# delete profiles

# first delete max and min profiles
delete NormOneMaxHeatStorCol, NormOneMaxHeatBoilOutCol, NormOneMaxPurCol, NormOneMaxBattCol;
delete NormOneMinHeatStorCol, NormOneMinHeatBoilOutCol, NormOneMinPurCol, NormOneMinBattCol;

# then delete norm-one profiles
delete NormOnePurProf, NormOneBattProf, NormOneHeatStorProf, NormOneHeatBoilOutProf;
# finally, delete pool profiles
delete PurProfPool, BattProfPool, BattIOProfPool, HeatStorProfPool, HeatStorOutProfPool, HeatBoilOutProfPool;

# delete sets
delete BattColPool, BattIOColPool, HeatStorColPool, HeatStorOutColPool, PurColPool, HeatBoilOutColPool;
delete NormOneMaxGenCol, NormOneMinGenCol;
#delete OnOffColPool, GenColPool;

##############################################################################
