####################################################################################
# Solve the LP relaxation of the coarse MILP model of distributed generation (DG)  #
# AMPL coding by                                              			   #
# Fu Lin                                                    			   #
# Argonne National Laboratory, 2014                           			   #
####################################################################################

#option LP_OutFile ("results/" & $Building & "/CoarseModelLP_" & $HorizonDays   & "day.out");
#option LP_log_file ("results/" & $Building & "/CoarseModelLP_" & $HorizonDays  & "day.log");

option LP_OutFile ("results/" & $Building & "/CoarseModelLP_" & $HorizonDays   &  "day"  &  $Method  & ".out");
option LP_log_file ("results/" & $Building & "/CoarseModelLP_" & $HorizonDays  &  "day"  &  $Method  & ".log");

### solve LP-relaxation ###

param LPMaxIter := 100;					# ... maximum number of iterations for solving coarse model
param LPAddCon{1..LPMaxIter} default 0;			# ... number of violated constraints added to coarse model


solve > ($LP_log_file); 
display  numTech, UpTech > ($LP_log_file); 
let LPAddCon[1] := _ncons - _ncons_csm; 
let total_solve_time := total_solve_time + _solve_time; 
printf '%13s%15s%15s%15s%15s%15s%15s\n', 'Year/Model', 'bVars', 'cVars', 'cons', 'Time', 'FirstStageSol', 'ObjVal' 
	> ($LP_OutFile);

printf '%2i/Coar-Iter1%15i%15i%15i%15.2f (%3.2f,%3.2f,%3.2f,%3.2f,%3.2f) %.8e\n', 
	num($HorizonDays)/364, _snbvars, _nvars-_snbvars, _ncons, _solve_time, 
	numTech['Battery'], numTech['Boiler'], numTech['ChpSOFC'], 
	numTech['PowerSOFC'], numTech['Storage'], TotalCost 
	> ($LP_OutFile);

     for {lpr in 2..LPMaxIter}{

	   restore {d in Days, h in Hours: 
		HourlyPowerDemandConstr[d,h].body <= HourlyPowerDemandConstr[d,h].lb - con_eps 
		or 
		HourlyPowerDemandConstr[d,h].body >= HourlyPowerDemandConstr[d,h].ub + con_eps} 
		HourlyPowerDemandConstr[d,h];

	   restore {d in Days, h in Hours: 
		HourlyPowerStorageBound[d,h].body <= HourlyPowerStorageBound[d,h].lb - con_eps
		or 
		HourlyPowerStorageBound[d,h].body >= HourlyPowerStorageBound[d,h].ub + con_eps} 
		HourlyPowerStorageBound[d,h];

	   restore {d in Days, h in Hours: 
		HourlyHeatDemandConstr[d,h].body <= HourlyHeatDemandConstr[d,h].lb - con_eps
		or
		HourlyHeatDemandConstr[d,h].body >= HourlyHeatDemandConstr[d,h].ub + con_eps} 
			HourlyHeatDemandConstr[d,h];

	   restore {d in Days, h in Hours: 
		HourlyHeatOutBoilerBound[d,h].body <= HourlyHeatOutBoilerBound[d,h].lb - con_eps
		or
		HourlyHeatOutBoilerBound[d,h].body >= HourlyHeatOutBoilerBound[d,h].ub + con_eps} 
		HourlyHeatOutBoilerBound[d,h];

	   restore {d in Days, h in Hours: 
		HourlyHeatStorageBound[d,h].body <= HourlyHeatStorageBound[d,h].lb - con_eps
		or
		HourlyHeatStorageBound[d,h].body >= HourlyHeatStorageBound[d,h].ub + con_eps} 
		HourlyHeatStorageBound[d,h];

	   restore {d in Days, h in 1..23: 
		HourlyHeatInBound[d,h].body <= HourlyHeatInBound[d,h].lb - con_eps
		or
		HourlyHeatInBound[d,h].body >= HourlyHeatInBound[d,h].ub + con_eps} 
		HourlyHeatInBound[d,h];

	   restore {d in Days, h in Hours, j in TechsSOFC: 
		HourlyOnUnitsBound[d,h,j].body <= HourlyOnUnitsBound[d,h,j].lb - con_eps
		or 
		HourlyOnUnitsBound[d,h,j].body >= HourlyOnUnitsBound[d,h,j].ub + con_eps} 
		HourlyOnUnitsBound[d,h,j];

	   restore {p in Periods, t in StartPeriod[p]..EndPeriod[p]: 
		HourlyMaxUtility[p,t].body <= HourlyMaxUtility[p,t].lb - con_eps
		or
		HourlyMaxUtility[p,t].body >= HourlyMaxUtility[p,t].ub + con_eps} 
		HourlyMaxUtility[p,t];        

	   restore {d in Days, h in 1..24, j in TechsSOFC, i in Units[j] diff {last(Units[j])}: 
		HourlySymmBreak[d,h,j,i].body <= HourlySymmBreak[d,h,j,i].lb - con_eps
		or
		HourlySymmBreak[d,h,j,i].body >= HourlySymmBreak[d,h,j,i].ub + con_eps} 
		HourlySymmBreak[d,h,j,i];

	  
	   let LPAddCon[lpr] := _ncons - _ncons_csm;  

	   # if the number of added constraints are the same for the last two iterates
	   # then stop re-solving coarse model
	   if (lpr>=2) and (LPAddCon[lpr] = LPAddCon[lpr-1])
	   then {printf "No more constraints added. The number of solved coarse model is %2i. \n", lpr-1 > ($LP_OutFile);
		break;}

	   # re-solve the coarse model with added constraints
	   solve > ($LP_log_file); 
	   let total_solve_time := total_solve_time + _solve_time;
	   display numTech, UpTech > ($LP_log_file); 

	   printf "%2i/Coar-Iter%1i%15i%15i%15i%15.2f (%3.2f,%3.2f,%3.2f,%3.2f,%3.2f) %.8e\n", 
		num($HorizonDays)/364, lpr, _snbvars, _nvars-_snbvars, _ncons, _solve_time, 
		numTech['Battery'], numTech['Boiler'], numTech['ChpSOFC'], numTech['PowerSOFC'], numTech['Storage'], TotalCost 
		> ($LP_OutFile);
	   
    }

