#############################################################
# Solve the full MILP model of distributed generation (DG)  #
# AMPL coding by                                            #
# Fu Lin                                                    #
# Argonne National Laboratory, 2014                         #
#############################################################

### OPTIONS ###
option solver cplexamp;
option cplex_options 'mipdisplay=2' 'mipinterval=1000' 'mipgap=1.0e-02' 'nodefile=2' 'time=10800' 'threads=1' 'prestats 1';
option show_stats 1;         
option DatFile ("data/" & $Building & "/" & $HorizonDays & "day.dat");
option log_file ("results/" & $Building & "/FullModel_" & $HorizonDays & "day.log");
option OutFile ("results/" & $Building & "/FullModel_" & $HorizonDays  & "day.out");

### load model and data ####
model Full.mod;
data data/DG.dat;
data ($DatFile);

### solve the model and save results ###
solve;
display _solve_time;
display numTech, UpTech;
display _snbvars, _nvars-_snbvars, _ncons;
printf "Objective = %12.9f\n\n\n", TotalCost;

printf '%13s%15s%15s%15s%15s%15s%15s\n', 'Year/Model', 'bVars', 'cVars', 'cons', 'Time', 'FirstStageSol', 'ObjVal' 
	> ($OutFile);

printf '%2i/Full%15i%15i%15i%15.2f(%2i,%2i,%2i,%2i,%2i)%15.2f\n', 
	num($HorizonDays)/364, _snbvars, _nvars-_snbvars, _ncons, _solve_time, 
	numTech['Battery'], numTech['Boiler'], numTech['ChpSOFC'], 
	numTech['PowerSOFC'], numTech['Storage'], TotalCost 
	> ($OutFile);