####################################################################
# Solve the semi-coarse MILP model of distributed generation (DG)  #
# AMPL coding by                                                   #
# Fu Lin                                                           #
# Argonne National Laboratory, 2014                                #
####################################################################

### OPTIONS ###
# crank crush steamroller grind churn
# Hospital School Retail Restaurant SuperMarket
#option Building School;

#option Building Hospital;
#option HorizonDays 364;
#option NumDays 4;		
#option Method Sampling;		# ... Sampling or Kmeans

option solver cplexamp;
option cplex_options 'mipdisplay=2' 'mipinterval=1000' 'mipgap=1.0e-02' 'nodefile=2' 'time=10800' 'threads=1' 'prestats 1';
option show_stats 1;         

option MovHor MovHorWinLen4;
option OutFile ("results/" & $Building & "/SemiCoarseModel_" & $HorizonDays  &  "day"  &  $Method  & ".out");
option log_file ("results/" & $Building &"/SemiCoarseModel_" & $HorizonDays  &  "day"  &  $Method  & ".log");
option DatFile ("data/" & $Building & "/" & $HorizonDays & "day.dat");

### load model and data ###
model SemiCoarse.mod;
model CheckProf.mod;
data  data/DG.dat;
data ($DatFile);

######################################################
	commands ProfLoad.ampl;
######################################################

######################################################

if $Method = "Sampling" 
   then	{commands ProfSelectSampling.ampl;}
   else if $Method = "Kmeans" 
        then {commands ProfSelectKmean.ampl;}
        else printf "Method for profile selection is either Sampling or Kmeans.";

######################################################

solve;  
display _solve_time;
display numTech, UpTech;
display _snbvars, _nvars-_snbvars, _ncons;
printf "Objective = %12.9f\n\n\n", TotalCost; 

printf '%13s%15s%15s%15s%15s%15s%15s\n', 'Year/Model', 'bVars', 'cVars', 'cons', 'Time', 'FirstStageSol', 'ObjVal' 
	> ($OutFile);

printf '%2i/Semi%15i%15i%15i%15.2f (%2i,%2i,%2i,%2i,%2i) %15.2f\n', 
	num($HorizonDays)/364, _snbvars, _nvars-_snbvars, _ncons, _solve_time, 
	numTech['Battery'], numTech['Boiler'], numTech['ChpSOFC'], 
	numTech['PowerSOFC'], numTech['Storage'], TotalCost 
	> ($OutFile);
