#####################################################################################
#!/bin/sh		
# Generate tables from results in .log and .out files for Distributed Generation
# shell script coding	
# by Fu Lin
# Argonne National Laboratory, 2014
#####################################################################################
# Instructions:
# 1) Specify environment variable model="Full" or "SemiCoarse"
# 2) If model="Full", uncomment line "for HorizonDay in 4 7 14 28 84 364" below
# 3) If model="SemiCoarse", then need to specify Method="Sampling" or "Kmeans"
# 		      uncomment line "for HorizonDay in 364 728 1456 2184 2912 3640"
# The default setting is specified in the "runTest.sh", which passes the environment
# variables: model, Method and Test into this shell script.
#####################################################################################

#model="Full";
#model="SemiCoarse";

#Method="Sampling";
#Method="Kmeans";

table=$model"Model.tab";
tempfile="temp.txt";

# create a table format for the data
printf "%11s%10s%5s%10s%10s%10s%8s%10s%20s%12s%18s%5s\n" "Building" "StopTol" "Days" "Bin." "Cont." "Constr." "Nodes" "LP-iter" "FirstStageSol" "Time" "ObjectiveValue" "MSG"> $table;


for Building in  School SuperMarket Restaurant Retail  Hospital
    do
    for HorizonDay in 4 				      # ... use this line for test cases
    #for HorizonDay in 4 7 14 28 84 364                       # ... use this line for full models
    #for HorizonDay in 364 728 1456 2184 2912 3640            # ... use this line for semicoarse models
         do

	 if [ "$model" = "Full" ];
	 then
		 logfile=$Building"/"$model"Model_"$HorizonDay"day.log";
		 outfile=$Building"/"$model"Model_"$HorizonDay"day.out";
	 fi

	 if [ "$model" = "SemiCoarse" ];
	 then
		 logfile=$Building"/"$model"Model_"$HorizonDay"day"$Method".log";
		 outfile=$Building"/"$model"Model_"$HorizonDay"day"$Method".out";
	 fi

	 # print out the building
	 printf "%11s" $Building >> $table;

	 # get the solve message into a separate file
	 (grep  -A8 "Total" $logfile) > $tempfile

	 # print out the stopping criterion
	 Temp=$(grep "relmipgap" $tempfile)
	 Temp=${Temp#* =}    		# ... remove everything before "="
	 Temp=${Temp#* =}    		# ... remove everything before "="
	 #Temp=${Temp%\\*}    		# ... remove everything after "\"
	 printf "  %.2e" $Temp >> $table
	 #echo $Temp

	  # print out number of days, stopping criterion, and dataset to the table
	 printf "%5s" $HorizonDay >> $table;

	 # print out the number of binary variables
	 Temp=$(grep "_snbvars" $logfile);
	 Temp=${Temp#*= };
	 Temp=${Temp%_nvars*};
	 printf "%10s" $Temp >> $table;

	 # print out the number of continuous variables
	 Temp=$(grep "_nvars" $logfile);
	 Temp=${Temp#*=};
	 printf "%10s" $Temp >> $table;

	 # print out the number of constraints 
	 Temp=$(grep "_ncons" $logfile);
	 Temp=${Temp#*= };
	 printf "%10s" $Temp >> $table;

	 # print out the number of nodes to table
	 Temp=$(grep "branch-and-bound nodes" $tempfile);
	 Temp=${Temp% br*};
       	 printf "%8s" $Temp >> $table;
	
	 # print out the number of simplex iterations
	 Temp=$(grep "MIP simplex iterations" $tempfile);
	 Temp=${Temp% MIP*};
       	 printf "  %.2e" $Temp >> $table;
	 #echo $Temp

	 # get the first-stage solutions from the outfile
	 #Temp=$(grep "optimal integer solution" $logfile);
	 #if [[ $Temp == *"optimal integer solution"* ]]
	 #then
	     # extract the first-stage solution from .out file
	     Temp=$(grep "(" $outfile);
	     Temp=${Temp#*(}
	     Temp=${Temp%)*}
	 #else 
	 #    Temp=""
 	 #fi

         printf "%20s" "$Temp" >> $table;

	 # find the line starting with "_solve_time" and put that line into a variable
	 Temp=$(grep "_solve_time" $logfile);

	 # remove everything up to an equal sign
	 Temp=${Temp#*=}

         # print out the number of seconds to table

	 if [ "$Test" = "True" ];
	 then
	 printf "    %.2e" "" >> $table;
	 else
	 printf "    %.2e" "$Temp" >> $table;
	 fi

	 # find the objective value 
	 Temp=$(grep "Objective" $logfile);

	 # remove everything up to an equal sign
	 Temp=${Temp#*=}

         # print out the objective value to table
	 printf "    %.8e" "$Temp" >> $table;

	 # get the error message
	 cplexMsg=$(grep "CPLEX 12.6.1.0" $tempfile);
	 case "$cplexMsg" in 
	     *"optimal"*"integer solution"*) MSG="0"
	     ;;
	     *"time limit with integer solution"*) MSG="1"
	     ;;
	     *"out of memory"*) MSG="2"
	     ;;
	     *"unrecoverable failure"*) MSG="3"
	     ;;
	 esac

	 # print the message code
	 printf "%5s\n" $MSG >> $table;	 

	 echo "Print out results for $Building in $HorizonDay day example... Done."
    done
done

rm $tempfile

printf "\n\n" >> $table;
printf "MSG   Meanings\n" >> $table;
printf "0     Integer solution within mipgap or absmipgap\n" >> $table;
printf "1     Time limit with integer solution\n" >> $table;
printf "2     Out of memory\n" >> $table;
printf "3     Unrecoverable failure\n" >> $table;
