#!/bin/octave
###############################################
# Choose profiles using clustering algorithms #
# AMPL coding by                              #
# Fu Lin                                      #
# Argonne National Laboratory, 2014           #
###############################################

# get the environment variables specified in "runClusterProf.sh"

dirname=getenv("dirname");
filename=getenv("filename");
clusternum=getenv("clusternum");
NumDays=getenv("NumDays");
days=str2num(NumDays);

# show plots or not (PlotSwitch = 1 yes show plots)
PlotSwitch = 0;

# load the snapshot file
data = load([dirname,'/',filename,'.sns']);

# put the data into a matrix of dimension 24-by-days
A = reshape(data,24,days);
absumA = sum(abs(A));

# load the statistics package
# pkg load statistics;

# choose the number of clusters and run kmeans algorithm
k = str2num(clusternum);

if sum(absumA) >= k
	# ... if the number of nonzero profiles is no smaller than k 
	# ... then run kmeans algorithm

  [idx,ctr,flag] = mykmeans(A',k);      

	# ... idx is the indices of each profile belonging to one of k clusters
	# ... ctr is the cluster centers, which are not profiles
	# ... therefore, we need to find the closest profiles to centers

	# ... find the indices of cluster center
	if (flag==1)
	    ctridx = ones(1,k);
	    ctrdist = zeros(days,k);
	    for i = 1:k
		for j = 1:days
		     ctrdist(j,i) = norm( A(:,j) - ctr(i,:)' );
		endfor
		[mindist, ctridx(i)] = min(ctrdist(:,i));
	    endfor
	endif

	if (flag==0)
	      ctridx = [1:1:k];
	endif

elseif sum(absumA) > 0

	# ... if not all profiles are zero, then choose nonzero profiles as clusters 
	  ctridx = ones(1,k);
	ctridx(1:sum(absumA)) = find(absumA);
	disp(['There are only ',num2str(sum(find(absumA))),' nonzero profiles.']);
	
	# ... if all profiles are zero, then choose one zero profile as the only cluster
	  else ctridx=[1:1:k];
	disp(['All profiles are zero. Only one cluster is generated.']);
endif


# plot profiles
close all;
if (PlotSwitch)
    for i=1:k
      figure(i)
      plot(A(:,idx==i));
      figure(i+100)
      plot(ctr(i,:));
      figure(i+200)
      plot(A(:,ctridx(i)));
    endfor
endif

# save the indices for the clusters
fileName = [dirname,'/',filename,'kmeans',num2str(k),'.idx'];
[fileID, MSG] = fopen(fileName,'w');

# print indices of cluster centers to file
fprintf(fileID, '%5i\n', ctridx(:));
fclose(fileID);

# save the cluster center profiles to file
# fileName = [dirname,'/',filename,'kmeans',num2str(k),'.ctr'];
# [fileID, MSG] = fopen(fileName,'w');

# print cluster center to file
# fprintf(fileID, '%15.5f%15.5f\n', A(:,ctridx)');
# fclose(fileID);
